#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using xMessages.TextEffects;
using xMessages.ImageEffects;
using xMessages.Animation;
#endregion

namespace xMessages_Tester
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        private xMessages.MessageComponent talk;
        private List<xMessages.MessageInformation> textItems = new List<xMessages.MessageInformation>();
        private IUserInputService input;
        private int currentIndex = 0;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
            this.Services.AddService(typeof(ContentManager), this.content);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            this.talk = new xMessages.MessageComponent(this);
            this.Components.Add(this.talk);
            this.input = new UserInputService(this);
            this.Services.AddService(typeof(IUserInputService), this.input);
            this.IsMouseVisible = true;
            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                // TODO: Load any ResourceManagementMode.Automatic content
            }

            // TODO: Load any ResourceManagementMode.Manual content
            //this.talk.SkinTexture = content.Load<Texture2D>("Dialog");

            SpriteFont fnt = content.Load<SpriteFont>("Courier New");
            Texture2D img = content.Load<Texture2D>("PlantMonster");
            Rectangle r = new Rectangle(0, 0, img.Width, img.Height);

            AddMessages(fnt, img, ref r);

            // load the panels from the file
            System.IO.Stream data = content.Load<System.IO.Stream>("Panel");
            List<xMessages.MessageControl> panels = xFripperyXNA.GUIReader.Load<xMessages.MessageControl>(data);
            Helpers.ExtractData(panels[0], true, this);

            this.talk.Control = panels[0];
            this.talk.ContentControlName = "center";
            this.talk.DestinationRectangle = new Rectangle(80, 60, 640, 480);

            // add event handlers
            for (int idx = 0; idx <= this.textItems.Count - 1; idx++)
            {
                this.textItems[idx].Ended += new xMessages.MessageInformation.EndedEventHandler(this.TextMsgEnded);
            }
            this.talk.QueueMessage(this.textItems[0]);
        }

        private void AddMessages(SpriteFont fnt, Texture2D img, ref Rectangle r)
        {
            TimeSpan totalDisplayTime = TimeSpan.FromSeconds(5);

            // add first message  =======================================================
            System.Collections.Specialized.StringCollection msg = new System.Collections.Specialized.StringCollection();
            msg.AddRange(new string[] { "This is the xMessage component.", "", "Press spacebar or button A to continue. Or you can wait 5 seconds." });
            xMessages.MessageInformation t = new xMessages.MessageInformation(msg, fnt, img.Width, img.Height, totalDisplayTime);
            using (PlainImage imageFX = t.GetImageEffect<PlainImage>())
            {
                imageFX.Image = img;
                imageFX.ImageSourceRectangle = r;
            }
            this.textItems.Add(t);

            // add second message  ===============================================
            msg = new System.Collections.Specialized.StringCollection();
            msg.AddRange(new string[] { "The xMessage component allows for text based character dialog to be displayed similar to classic games like Final Fantasy for the NES.", 
                                        "", "Press spacebar or button A to continue."});
            totalDisplayTime = TimeSpan.Zero;
            t = new xMessages.MessageInformation(msg, fnt, img.Width, img.Height, totalDisplayTime);
            using (PlainImage imageFX = t.GetImageEffect<PlainImage>())
            {
                imageFX.Image = img;
                imageFX.ImageSourceRectangle = r;
            }
            xMessages.TextEffects.TypedText typedEffect = new xMessages.TextEffects.TypedText();
            typedEffect.TextSpeed = TimeSpan.FromMilliseconds(5);
            t.TextEffect = typedEffect;
            this.textItems.Add(t);

            // add third message =================================================
            msg = new System.Collections.Specialized.StringCollection();
            msg.AddRange(new string[] { "The xMessage component also supports messages with no image.", "", "Press spacebar or button A to continue." });
            t = new xMessages.MessageInformation(msg, fnt, totalDisplayTime);
            using (PlainImage imageFX = t.GetImageEffect<PlainImage>())
            {
                imageFX.Image = img;
                imageFX.ImageSourceRectangle = r;
            }
            this.textItems.Add(t);

            // add messages 4 to 7 =================================================
            foreach (xMessages.MessageInformation.ImageDisplayPosition pos in Enum.GetValues(typeof(xMessages.MessageInformation.ImageDisplayPosition)))
            {
                msg = new System.Collections.Specialized.StringCollection();
                msg.AddRange(new string[] { "ImagePosition property is set to " + pos.ToString(), "", "Press spacebar or button A to continue." });
                t = new xMessages.MessageInformation(msg, fnt, img.Width, img.Height, totalDisplayTime);
                using (PlainImage imageFX = t.GetImageEffect<PlainImage>())
                {
                    imageFX.Image = img;
                    imageFX.ImageSourceRectangle = r;
                }
                t.ImagePosition = pos;
                this.textItems.Add(t);
            }

            // add 8th message  =================================================
            totalDisplayTime = TimeSpan.FromSeconds(4);
            msg = new System.Collections.Specialized.StringCollection();
            msg.AddRange(new string[] { "You can also set the text color.", "", "Press spacebar or button A to continue." });
            t = new xMessages.MessageInformation(msg, fnt, totalDisplayTime);
            using (PlainImage imageFX = t.GetImageEffect<PlainImage>())
            {
                imageFX.Image = img;
                imageFX.ImageSourceRectangle = r;
            }
            using (PlainText textFX = t.GetTextEffect<PlainText>())
            {
                textFX.TextColor = Color.Red;
            }

            this.textItems.Add(t);

            // add 9th message =================================================
            msg = new System.Collections.Specialized.StringCollection();
            msg.AddRange(new string[] { "You can also set the image color.", "", "Press spacebar or button A to continue." });
            t = new xMessages.MessageInformation(msg, fnt, img.Width, img.Height, totalDisplayTime);
            using (PlainImage imageFX = t.GetImageEffect<PlainImage>())
            {
                imageFX.Image = img;
                imageFX.ImageSourceRectangle = r;
                imageFX.ImageColor = Color.Red;
            }
            this.textItems.Add(t);


            // add 10th message =================================================
            msg = new System.Collections.Specialized.StringCollection();
            string repeatText = "Sample Text, ";
            for (int idx = 0; idx < 5; idx++)
            {
                repeatText += repeatText;
            }
            repeatText = repeatText.Remove(repeatText.Length - 2);

            msg.AddRange(new string[] { "If too much text is being displayed you can specify the speed at which to scroll the text.", 
                                        "", repeatText, "", "Press spacebar or button A to continue." });
            totalDisplayTime = TimeSpan.Zero;
            t = new xMessages.MessageInformation(msg, fnt, img.Width, img.Height, totalDisplayTime);
            xMessages.TextEffects.PlainText tfx = t.GetTextEffect<xMessages.TextEffects.PlainText>();
            tfx.DelayAtStart = TimeSpan.FromSeconds(2);   // when message is displayed wait 2 secs before starting to scroll
            tfx.TextSpeed = TimeSpan.FromMilliseconds(5); // lower values
            using (PlainImage imageFX = t.GetImageEffect<PlainImage>())
            {
                imageFX.Image = img;
                imageFX.ImageSourceRectangle = r;
            }
            this.textItems.Add(t);


            // add 11th message =================================================
            msg = new System.Collections.Specialized.StringCollection();
            msg.AddRange(new string[] { "Hello. I am a talking head!", 
                                        "", "Pretty neat huh?", "", "Press spacebar or button A to continue." });
            totalDisplayTime = TimeSpan.Zero;
            t = new xMessages.MessageInformation(msg, fnt, img.Width, img.Height, totalDisplayTime);
            // set the text effect
            typedEffect = new xMessages.TextEffects.TypedText();
            typedEffect.TextSpeed = TimeSpan.FromMilliseconds(5);
            t.TextEffect = typedEffect;

            // set the image effect
            AnimatedImage aniImageFX = new AnimatedImage();
            t.ImageEffect = aniImageFX;

            // add main animation
            AnimationInfo info = new AnimationInfo(0, 1, 10, true);
            aniImageFX.Animation.AddAnimation("main", info);
            // construt frames for main animation
            SpriteFrame[] frames = new SpriteFrame[2];
            // frame 1
            frames[0] = new SpriteFrame( );
            frames[0].Texture = content.Load<Texture2D>("jungefrau");
            frames[0].SourceRectangle = new Rectangle(0, 0, frames[0].Texture.Width, frames[0].Texture.Height);
            // frame 2
            frames[1] = new SpriteFrame();
            frames[1].Texture = content.Load<Texture2D>("jungefrau2");
            frames[1].SourceRectangle = new Rectangle(0, 0, frames[1].Texture.Width, frames[1].Texture.Height);

            aniImageFX.Animation.Frames.AddRange(frames);
            aniImageFX.CurrentAnimation = "main";
            this.textItems.Add(t);

            // add 12th message =================================================
            msg = new System.Collections.Specialized.StringCollection();
            msg.AddRange(new string[] { "Hello. I am another talking head!", 
                                        "", "As you can see the xMessage compoent allows for some simple character dialog.", "", "Press spacebar or button A to continue." });
            totalDisplayTime = TimeSpan.Zero;
            t = new xMessages.MessageInformation(msg, fnt, img.Width, img.Height, totalDisplayTime);
            // set the text effect
            typedEffect = new xMessages.TextEffects.TypedText();
            typedEffect.TextSpeed = TimeSpan.FromMilliseconds(5);
            t.TextEffect = typedEffect;
            t.ImagePosition = xMessages.MessageInformation.ImageDisplayPosition.UpperRight;

            // set the image effect
            aniImageFX = new AnimatedImage();
            t.ImageEffect = aniImageFX;

            // add main animation
            info = new AnimationInfo(0, 1, 10, true);
            aniImageFX.Animation.AddAnimation("main", info);
            // construt frames for main animation
             frames = new SpriteFrame[2];
            // frame 1
            frames[0] = new SpriteFrame();
            frames[0].Texture = content.Load<Texture2D>("jungermann");
            frames[0].SourceRectangle = new Rectangle(0, 0, frames[0].Texture.Width, frames[0].Texture.Height);
            // frame 2
            frames[1] = new SpriteFrame();
            frames[1].Texture = content.Load<Texture2D>("jungermann2");
            frames[1].SourceRectangle = new Rectangle(0, 0, frames[1].Texture.Width, frames[1].Texture.Height);

            aniImageFX.Animation.Frames.AddRange(frames);
            aniImageFX.CurrentAnimation = "main";
            this.textItems.Add(t);
        }

        private void TextMsgEnded(xMessages.MessageInformation e)
        {
            this.currentIndex++;
            if (this.currentIndex > this.textItems.Count - 1) this.currentIndex = 0;
            this.talk.QueueMessage(this.textItems[this.currentIndex]);
        }

        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent)
            {
                // TODO: Unload any ResourceManagementMode.Automatic content
                content.Unload();
            }

            // TODO: Unload any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            this.input.Update(gameTime);

            // Allows the game to exit at any time
            if (this.input.KeyClick(Keys.Escape) || this.input.AClick(PlayerIndex.One)) this.Exit();

            // handles input for each dialog message
            HandleDialogInput();

            // TODO: Add your update logic here
            if (this.input.IsRightMouseButtonDown == true)
            {
                this.talk.Width += this.input.RelativeMouseX;
                this.talk.Height += this.input.RelativeMouseY;
            }

            if (this.input.IsLeftMouseButtonDown == true)
            {
                this.talk.Left += this.input.RelativeMouseX;
                this.talk.Top += this.input.RelativeMouseY;
            }

            base.Update(gameTime);
        }

        private void HandleDialogInput()
        {
            if (this.input.KeyClick(Keys.Space) || this.input.AClick(PlayerIndex.One))
            {
                this.talk.DequeueMessage();
            }
            return;
            switch (this.currentIndex)
            {
                case 0:
                    if (this.input.KeyClick(Keys.Space) || this.input.AClick(PlayerIndex.One))
                    {
                        this.talk.DequeueMessage();
                    }
                    break;

                case 1:
                    if (this.input.KeyClick(Keys.Space) || this.input.AClick(PlayerIndex.One))
                    {
                        this.talk.DequeueMessage();
                    }
                    break;

                case 2:
                    if (this.input.KeyClick(Keys.Space) || this.input.AClick(PlayerIndex.One))
                    {
                        this.talk.DequeueMessage();
                    }
                    break;
            }
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
