using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using xMessages_Tester.Properties;


namespace xMessages_Tester
{
    public static class Helpers
    {
        public static void ExtractData(xMessages.MessageControl c, bool children)
        {
            ExtractData(c, children, null);
        }

        public static void ExtractData(xMessages.MessageControl c, bool children, Game game)
        {
            ContentManager content = (ContentManager)game.Services.GetService(typeof(ContentManager));
            System.Xml.XmlDocument doc = new System.Xml.XmlDocument();
            doc.LoadXml("<data>" + c.Data + "</data>");
             
            System.Xml.XmlNode node;
            node = doc.DocumentElement.SelectSingleNode(Resources.XML_texture);

            // check if data contains a texture element
            if (node != null && node.InnerText != string.Empty)
            {
                c.Texture = content.Load<Texture2D>(node.InnerText);
            }

            Rectangle sr = c.SourceRectangle;
            node = doc.DocumentElement.SelectSingleNode(Resources.XML_sourceleft);

            // extract source texture rect
            if (node != null && !int.TryParse(node.InnerText, out sr.X))
            {
                // do nothing?
            }
            node = doc.DocumentElement.SelectSingleNode(Resources.XML_sourcetop);
            if (node != null && !int.TryParse(node.InnerText, out sr.Y))
            {
                // do nothing?
            }
            node = doc.DocumentElement.SelectSingleNode(Resources.XML_sourcewidth);
            if (node != null && !int.TryParse(node.InnerText, out sr.Width))
            {
                // do nothing?
            }
            node = doc.DocumentElement.SelectSingleNode(Resources.XML_sourceheight);
            if (node != null && !int.TryParse(node.InnerText, out sr.Height))
            {
                // do nothing?
            }
            // set source rect
            c.SourceRectangle = sr;

            // read color data
            node = doc.DocumentElement.SelectSingleNode(Resources.XML_color);
            if (node != null)
            {
                Color col = c.Color;
                if (ParseColor(node, out col)) c.Color = col;
            }


            doc = null;

            // check is wanting to extract child data as well
            if (children)
            {
                // extract data for children also
                foreach (xMessages.MessageControl child in c.Children)
                {
                    // if (child.GetType() == typeof(Interfaces.IGUIWidget))
                    // {
                    // Classes.GUIWidget gw = (GUIWidget)child;
                    // gw.ExtractData(children);
                    Helpers.ExtractData(child, children, game);
                    //}
                }
            }


        }

        private static bool ParseColor(System.Xml.XmlNode node, out Color c)
        {
            c = Color.White;
            System.Xml.XmlNode rn, gn, bn, an;
            rn = node.Attributes.GetNamedItem(Resources.XML_r);
            gn = node.Attributes.GetNamedItem(Resources.XML_g);
            bn = node.Attributes.GetNamedItem(Resources.XML_b);
            an = node.Attributes.GetNamedItem(Resources.XML_a);

            byte rv = 255;
            byte gv = 255;
            byte bv = 255;
            byte av = 255; // set default alpha value

            // extract color components
            if (rn != null && !byte.TryParse(rn.InnerText, out rv))
            {
                return false;
            }
            if (gn != null && !byte.TryParse(gn.InnerText, out gv))
            {
                return false;
            }
            if (bn != null && !byte.TryParse(bn.InnerText, out bv))
            {
                return false;
            }
            if (an != null && !byte.TryParse(an.InnerText, out av))
            {
                // we can ignore alpha
            }

            // assign color
            c = new Color(rv, gv, bv, av);
            return true;
        }
    }
}
