/*
Created by: X
http://www.createdbyx.com/

xMessages is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace xMessages
{
    internal static class General
    {
        internal static void DrawString(SpriteBatch sb, SpriteFont fnt, System.Collections.Specialized.StringCollection text, Rectangle r, Color col)
        {
            General.DrawString(sb, fnt, text, r, col, true);
        }

        internal static void DrawString(SpriteBatch sb, SpriteFont fnt, System.Collections.Specialized.StringCollection text, Rectangle r, Color col, bool performWordWrap)
        {

            // check if there is text to draw
            if (text == null && text.Count == 0) return;
            System.Collections.Specialized.StringCollection lines = new System.Collections.Specialized.StringCollection();

            // duplicate the collection
            lock (text.SyncRoot)
            {
                foreach (string item in text)
                {
                    lines.Add(item);
                }
            }

             General.ProcessLines(fnt, r, performWordWrap, lines);

            // draw each line of text
            Vector2 pos = new Vector2(r.X, r.Y);
            for (int idx = 0; idx < lines.Count; idx++)
            {
                // if we are going to start drawing below the rectangles boundary then we can exit
                if (r.Bottom - pos.Y < fnt.LineSpacing) break;
                // draw the line of text
                sb.DrawString(fnt, lines[idx], pos, col);
                // move to the next position to begin drawing the next line of text
                pos.Y += fnt.LineSpacing;
            }
        }

        internal static Rectangle ProcessLines(SpriteFont fnt, Rectangle r, bool performWordWrap, System.Collections.Specialized.StringCollection lines)
        {
            // llop through each line in the collection
            Rectangle bounds = r;
            bounds.Width = 0;
            bounds.Height = 0;
            int index = 0;
            bool lineInserted = false;
            while (index < lines.Count)
            {
                // get a line of text
                string linetext = lines[index];
                //measure the line of text
                Vector2 size = fnt.MeasureString(linetext);

                // check if the line of text is geater then then the rect we want to draw it inside of
                if (performWordWrap && size.X > r.Width)
                {
                    // find last space character in line
                    string endspace = string.Empty;
                    // deal with trailing spaces
                    if (linetext.EndsWith(" "))
                    {
                        endspace = " ";
                        linetext = linetext.TrimEnd();
                    }

                    // get the index of the last space character
                    int i = linetext.LastIndexOf(" ");
                    if (i != -1)
                    {
                        // if there was a space grab the last word in the line
                        string lastword = linetext.Substring(i + 1);
                        // move word to next line 
                        if (index == lines.Count - 1)
                        {
                            lines.Add(lastword);
                            lineInserted = true;
                        }
                        else
                        {
                            // prepend last word to begining of next line
                            if (lineInserted)
                            {
                                lines[index + 1] = lastword + endspace + lines[index + 1];
                            }
                            else
                            {
                                lines.Insert(index + 1, lastword);
                                lineInserted = true;
                            }
                        }

                        // crop last word from the line that is being processed
                        lines[index] = linetext.Substring(0, i + 1);

                    }
                    else
                    {
                        // there appear to be no space characters on this line s move to the next line
                        lineInserted = false;
                        size = fnt.MeasureString(lines[index]);
                        if (size.X > bounds.Width) bounds.Width = (int)size.X;
                        bounds.Height += (int)size.Y;  
                        index++;
                    }
                }
                else
                {
                    // this line will fit so we can skip to the next line
                    lineInserted = false;
                    size = fnt.MeasureString(lines[index]);
                    if (size.X > bounds.Width) bounds.Width = (int)size.X;
                    bounds.Height += (int)size.Y;
                    index++;
                }
            }

            return bounds;
        }
    }
}
