/*
Created by: X
http://www.createdbyx.com/

xMessages is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace xMessages.ImageEffects
{
    public class AnimatedImage : IImageEffect, IDisposable
    {
        private Animation.AnimationSet<Animation.SpriteFrame> animation = new xMessages.Animation.AnimationSet<xMessages.Animation.SpriteFrame>();
        private string currentAni = string.Empty;


        public string CurrentAnimation
        {
            get { return currentAni; }
            set
            {
                if (this.animation.animations.ContainsKey(value))
                {
                    currentAni = value;
                    this.animation.Play(value);
                }
                else
                {
                    this.currentAni = string.Empty;
                    this.animation.CurrentAnimationInfo = null;
                }    
            }
        }


        public Animation.AnimationSet<Animation.SpriteFrame> Animation
        {
            get { return this.animation; }
        }

        #region IImageEffect Members

        public void Reset()
        {
            // does nothing
            this.animation.Stop();
            this.animation.Play(this.currentAni);
        }

        public void Update(Microsoft.Xna.Framework.GameTime gameTime, MessageInformation e, Rectangle rect)
        {
            this.animation.Update(gameTime);
        }

        public void Draw(Microsoft.Xna.Framework.GameTime gameTime, MessageInformation e, Rectangle rect, SpriteBatch spriteBatch)
        {
            if (e == null) return;
            if (spriteBatch == null) return;
            // check if the currently displayed message has an image associated with it.
            Animation.SpriteFrame frame = this.animation.CurrentFrame;
            if (frame.Texture == null) return;
            spriteBatch.Draw(frame.Texture, rect, frame.SourceRectangle, frame.Color);
        }


        #endregion


        #region IDisposable Members

        public void Dispose()
        {
            this.animation = null;
        }

        #endregion
    }
}
