/*
Created by: X
http://www.createdbyx.com/

xMessages is licenced under the Creative Commons Attribution-NonCommercial-ShareAlike 2.5 Canada
http://creativecommons.org/licenses/by-nc-sa/2.5/ca/
*/
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace xMessages.TextEffects
{
    public class PlainText : ITextEffect, IDisposable
    {
        private Color textColor = Color.White;
        private TimeSpan textSpeed = TimeSpan.FromMilliseconds(50);
        private StringCollection textToDraw;
        private bool moving = false;
        private TimeSpan nextMoveTime = TimeSpan.Zero;
        private int distanceToMove;
        private int distance = 0;
        private Rectangle prevRect;
        private TimeSpan startDelay = TimeSpan.FromSeconds(1);


        public TimeSpan DelayAtStart
        {
            get { return startDelay; }
            set { startDelay = value; }
        }

        public TimeSpan TextSpeed
        {
            get { return textSpeed; }
            set { textSpeed = value; }
        }

        /// <summary>
        /// Gets/Sets the color that the message will be drawn as.
        /// </summary>
        public Color TextColor
        {
            get { return textColor; }
            set { textColor = value; }
        }

        #region ITextEffect Members

        public void Reset()
        {
            // does nothing
            this.moving = false;
            this.nextMoveTime = TimeSpan.Zero;
            this.distanceToMove = 0;
            this.distance = 0;

        }

        public void Update(Microsoft.Xna.Framework.GameTime gameTime, MessageInformation e, Rectangle rect)
        {
            if (this.nextMoveTime == TimeSpan.Zero)
            {
                this.nextMoveTime = gameTime.TotalGameTime + this.startDelay;
                this.textToDraw = new StringCollection();

                // duplicate the collection
                lock (e.text.SyncRoot)
                {
                    foreach (string item in e.text)
                    {
                        this.textToDraw.Add(item);
                    }
                }
                General.ProcessLines(e.font, rect, true, this.textToDraw);

                this.prevRect = rect;
            }

            if (gameTime.TotalGameTime >= this.nextMoveTime)
            {
                Rectangle r = rect;

                // check if the rectagle changed since first call
              //  if (this.prevRect.Width != rect.Width || this.prevRect.Height != rect.Height)
              //  {
                    System.Collections.Specialized.StringCollection lines = new System.Collections.Specialized.StringCollection();

                    // duplicate the collection
                    lock (e.text.SyncRoot)
                    {
                        foreach (string item in e.text)
                        {
                            lines.Add(item);
                        }
                    }

                    // process the lines
                    r = General.ProcessLines(e.font, rect, true, lines);


                    // set the text to draw
                    this.textToDraw = lines;
                //}

                // calc the distance to move
                this.distanceToMove = r.Height - rect.Height;
                if (this.distanceToMove < 0)
                {
                    this.distanceToMove = 0;
                    this.distance = 0;
                }

                // update text position
                this.distance += 1;// *gameTime.ElapsedGameTime.Milliseconds;
                if (this.distance > this.distanceToMove) { this.distance = this.distanceToMove; }

                // set the next move time
                this.nextMoveTime += this.textSpeed;
            }
        }

        public void Draw(Microsoft.Xna.Framework.GameTime gameTime, MessageInformation e, Rectangle rect, SpriteBatch spriteBatch)
        {
            if (e == null) return;
            if (spriteBatch == null) return;


            //  General.DrawString(spriteBatch, e.Font, this.TextToDraw, rect, this.textColor);


            if (this.textToDraw != null)
            {
                // draw each line of text
                Vector2 pos = new Vector2(rect.X, rect.Y);
                for (int idx = 0; idx < this.textToDraw.Count; idx++)
                {
                    // if we are going to start draing below the rectangles boundary then we can exit
                 //   if (rect.Bottom - pos.Y < e.font.LineSpacing) break;
                    // draw the line of text
                    if (pos.Y > rect.Y -e.font.LineSpacing)
                    {
                        spriteBatch.DrawString(e.font, this.textToDraw[idx], pos + new Vector2(0, -this.distance), this.textColor);
                    }
                    // move to the next position to begin drawing the next line of text
                    pos.Y += e.font.LineSpacing;
                }
            }
        }

        #endregion



        #region IDisposable Members

        public void Dispose()
        {
            //  throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
}
